import nltk

from xportage.reading_raw.reader import DataReader
import xportage.utils.tree as tree_utils


class SST(DataReader):
    def read(self, path):
        data = {}
        metadata = {}

        with open(path) as f:
            sofar = 0
            for line in f:
                old_line = line[:]
                line = line.replace(' 1\/2', '-1\/2')
                if line != old_line:
                    print('Warning: Changes made to data.')
                    print('Before: {}'.format(old_line.strip()))
                    print(' After: {}'.format(line.strip()))
                tr = nltk.Tree.fromstring(line.strip())
                words = tr.leaves()
                spans = tree_utils.tree_to_spans_for_nltk(tr)

                data.setdefault('example_id', []).append(sofar)
                data.setdefault('sentence', []).append(words)
                data.setdefault('labeled_spans', []).append(spans)

                sofar += 1

        dataset = {}
        dataset['data'] = data
        dataset['metadata'] = metadata

        return dataset



import json


class DataReader(object):
    def __init__(self):
        pass

    def read_as_columns(self, path):
        data = {}
        metadata = {}

        with open(path) as f:
            for line in f:
                ex = json.loads(line)
                for k, v in ex.items():
                    data.setdefault(k, []).append(v)

        dataset = {}
        dataset['data'] = data
        dataset['metadata'] = metadata

        return dataset

    def read_as_rows(self, path):
        data = {}
        metadata = {}

        with open(path) as f:
            for line in f:
                ex = json.loads(line)
                data.setdefault('examples', []).append(ex)

        dataset = {}
        dataset['data'] = data
        dataset['metadata'] = metadata

        return dataset



import json
import os

from xportage.default_reference import reference as default_reference
from xportage.reading_raw.get_reader import get_reader_cls


class XPortageHome(object):
    def __init__(self, home, cfg, force=False):
        self.home = home
        self.cfg_path = cfg
        self.cfg = self.setup_cfg(cfg, force=force)

    def setup_cfg(self, cfg, force=False):
        if not os.path.exists(cfg) or force:
            print('Initializing config file at {}'.format(cfg))
            os.makedirs(os.path.dirname(cfg), exist_ok=True)
            with open(cfg, 'w') as f:
                cfg_data = {}
                cfg_data['reference'] = default_reference
                f.write(json.dumps(cfg_data, sort_keys=True, indent=4))

        with open(cfg) as f:
            return json.loads(f.read())

    def print_configuration(self):
        log = 'XPORTAGE CONFIGURATION\n'
        log += '\n'
        log += '  HOME={}\n'.format(self.home)
        log += '  CFG={}\n'.format(self.cfg_path)
        print(log)

    def print_summary(self):
        directories = sorted([x for x in os.listdir(self.home) if os.path.isdir(os.path.join(self.home, x))])

        print('HOME DIRECTORIES\n')
        for x in directories:
            print(x)
        print('')

    def get_ref(self, ref_name):
        return self.cfg['reference']['sources'][ref_name]

    def download(self, ref=None, ref_name=None):
        if ref_name is not None:
            assert ref is None, 'Should only specify one of ref or ref_name.'
            ref = self.get_ref(ref_name)
        print('Downloading ref with name {}.'.format(ref['name']))
        tmp_dir = os.path.join(self.home, 'tmp')
        os.makedirs(tmp_dir, exist_ok=True)

        target = os.path.join(self.home, ref['name'] + '.raw')
        if os.path.exists(target):
            print('Already downloaded.')
            return

        # Download.
        os.system('cd {} && wget {}'.format(tmp_dir, ref['url']))

        # Unpack.
        my_tmp = 'my_tmp-{}'.format(ref['name'])
        os.system('cd {} && {}'.format(tmp_dir, ref['unpack'].format(tmp=my_tmp)))

        mv_from = os.path.join(tmp_dir, my_tmp)
        mv_to = target
        os.system('mv {} {}'.format(mv_from, mv_to))

        # Cleanup.
        os.system('cd {} && {}'.format(tmp_dir, ref['cleanup']))

    def preprocess(self, ref=None, ref_name=None):
        if ref_name is not None:
            assert ref is None, 'Should only specify one of ref or ref_name.'
            ref = self.get_ref(ref_name)
        print('Preprocessing ref with name {}.'.format(ref['name']))
        raw_dir = os.path.join(self.home, ref['name'] + '.raw')
        out_dir = os.path.join(self.home, ref['name'] + '.preprocess')
        os.makedirs(out_dir, exist_ok=True)

        reader = get_reader_cls(ref['name'])()

        for x in ref['splits']:
            path = os.path.join(raw_dir, x['path'])
            dataset = reader.read(path)

            out_data = os.path.join(out_dir, '{}.data.jsonl'.format(x['name']))
            print('writing {}'.format(out_data))
            with open(out_data, 'w') as f:
                keys = list(dataset['data'].keys())
                length = None
                for k in keys:
                    if length is None:
                        length = len(dataset['data'][k])
                    assert length == len(dataset['data'][k])
                for i in range(length):
                    ex = {}
                    for k in keys:
                        ex[k] = dataset['data'][k][i]
                    f.write(json.dumps(ex, sort_keys=True) + '\n')

            out_metadata = os.path.join(out_dir, '{}.metadata.json'.format(x['name']))
            print('writing {}'.format(out_metadata))
            with open(out_metadata, 'w') as f:
                f.write(json.dumps(dataset['metadata'], sort_keys=True, indent=4))



import json
import nltk


def tree_to_spans_for_nltk(tree):
    spans = []
    def helper(tr, pos=0, depth=0):
        if len(tr) == 1 and isinstance(tr[0], str):
            label = tr.label()
            size = 1
            spans.append((pos, size, label, depth))
            return size
        if len(tr) == 1:
            label = tr.label()
            size = helper(tr[0], pos, depth=depth+1)
            spans.append((pos, size, label, depth))
            return size
        size = 0
        for x in tr:
            xsize = helper(x, pos+size, depth=depth+1)
            size += xsize
        label = tr.label()
        spans.append((pos, size, label, depth))
        return size
    _ = helper(tree)
    return spans



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="xportage",
    version="0.0.3",
    description="Makes reading data easy.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/mrdrozdov/xportage",
    author="Andrew Drozdov",
    author_email="andrew@mrdrozdov.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(where='.'),
    include_package_data=True,
    install_requires=[
        "nltk"
    ],
    entry_points={
        "console_scripts": [
            "xportage=xportage.__main__:main",
        ]
    },
)


# __main__.py

import argparse
import os

from xportage import xportage as xp

HOME = os.path.expanduser('~')


def main(options):
    xph = xp.XPortageHome(options.home, options.cfg, force=options.force)
    xph.print_configuration()
    xph.print_summary()

    if options.download:
        xph.download(ref_name=options.name)

    if options.preprocess:
        xph.preprocess(ref_name=options.name)



if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('home', type=str, help='Path to XPORTAGE_HOME.')
    parser.add_argument('--cfg', default=os.path.join(HOME, '.xportage', 'cfg.json'), type=str, help='Path to XPORTAGE_CFG.')
    parser.add_argument('--name', default=None, type=str, help='Dataset name.')
    parser.add_argument('--force', action='store_true', help='If True, then always reset config.')
    parser.add_argument('--download', action='store_true', help='If set, then will download dataset using reference info.')
    parser.add_argument('--preprocess', action='store_true', help='If set, then will preprocess dataset using reference info.')
    options = parser.parse_args()

    main(options)



reference = {}
sources = {}

# SST
x = {}
x['name'] = 'sst'
x['url'] = 'https://nlp.stanford.edu/sentiment/trainDevTestTrees_PTB.zip'
x['unpack'] = 'unzip trainDevTestTrees_PTB.zip -d {tmp}'
x['cleanup'] = 'rm trainDevTestTrees_PTB.zip'
x['splits'] = [
    {'name': 'dev', 'path': 'trees/dev.txt'},
    {'name': 'test', 'path': 'trees/test.txt'},
    {'name': 'train', 'path': 'trees/train.txt'}
]
sources[x['name']] = x

reference['sources'] = sources
